#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>


/*Simulation Parameters*/
#define w_inhomogeneous   10          /*inhomogeneous width mimiking solvent fluctuations*/
#define T2                1.          /*homogeneous dephasing time*/
#define w0                1660        /*frequency without hydrogen bond*/
#define anharmonicity     16          /*anharmonicity of an isolated C=O vibration*/
#define hydrogenbondshift 30          /*shift upon hydrogen bonding*/ 
#define dt                0.2         /*timestep for Fouriertransforms*/
#define nt                32	      /*number timesteps; has to be a number 2^n*/
#define woff              15          /*offset frequency to center spectra as good as possible in window*/ 


/*other constants*/
#define nmax              50         /* maximal number peptide groups */  
#define PI                3.1415927
#define cw                0.188       /*transformation factor frequency from cm-1 into ps-1*/

typedef struct COMPLEX {float re,im;} complex;

/* Numerical Recipes Routines */
void nrerror(char error_text[]);
float *vector(long nl, long nh);
float **matrix(long nrl, long nrh, long ncl, long nch);
void free_matrix(float **m, long nrl, long nrh, long ncl, long nch);
void free_vector(float *v, long nl, long nh);
void free_imatrix(int **m,long nrl,long nrh,long ncl,long nch);
void free_f3tensor(float ***t, long nrl, long nrh, long ncl, long nch,
        long ndl, long ndh);
float ***f3tensor(long nrl, long nrh, long ncl, long nch, long ndl, long ndh);
int **imatrix(long nrl, long nrh, long ncl, long nch);
int *ivector(long nl, long nh);
void free_ivector(int *v, long nl, long nh);
complex *complexvector(long nl, long nh);
void free_complexvector(complex *v, long nl, long nh);
complex **complexmatrix(long nrl, long nrh, long ncl, long nch);
void four1(float data[], unsigned long nn, int isign);
void eigsrt(float d[], float **v, int n);
float gasdev(long *idum);
void tred2(float **a, int n, float d[], float e[]);
void tqli(float d[], float e[], int n, float **z);

/*My Routines */
void fft(complex *P, int n);
void fft2D(complex **R, int n);
void parametrizedB3LYP(float **F,float **x,float **y,float **z,int n);
float dehydral (float x1,float x2,float x3,float x4, float y1,float y2,float y3,float y4, float z1,float z2,float z3,float z4);
void Hydrogenbond(float **F,float **x,float **y,float **z,int n);
float length(float x,float y, float z);
int coordinate_read(float **x, float **y, float **z, char *protein);
void dipole_one_exciton(float **I,float **Q,float *vxd,float *vyd,float *vzd,int n);
void two_exciton_matrix(float **F2,float **F,int n,int n2);
void dipole_two_exciton(float ***I2,float **Q2,float **Q,float *vxd,float *vyd,float *vzd,int n2,int n);
void LinResponse(complex *S,float *D,float **I, int n);
void NonlinResponse(complex **Rr,complex **Rnr,float *D,float *D2,float **I,float ***I2,int n,int n2,float t2);
void write_spectra(complex *S,complex **Rr,complex **Rnr,char *protein);
void transition_charges(float **F1,float *vxd,float *vyd,float *vzd,float **x,float **y,float **z,int n);
void interaction(float **F1,float *vxd,float *vyd,float *vzd,float **x,float **y,float **z,int n,char *name);

/*for random generator*/
long idum=-1;

main(int iarg, char **input)
{
char protein[100];
float **x,**y,**z;
float *vxd,*vyd,*vzd;
complex *S,**Rr,**Rnr;
int n2,i,j,i1,i2,n,k,it1,it3;
float **H1,**H2,*D,*e,**I,*D2,***I2,*e2;
int istruct;

if(iarg>=2) strcpy(protein,input[1]); else {printf("specify a peptide name!\n");exit(1);}

/*Initialize matrices*/
  n=nmax;
  n2=n*(n+1)/2;
  x=matrix(1,n,1,5);y=matrix(1,n,1,5);z=matrix(1,n,1,5);
  vxd=vector(1,n);vyd=vector(1,n);vzd=vector(1,n);
  D=vector(1,n);
  e=vector(1,n);
  H1=matrix(1,n,1,n);
  H2=matrix(1,n2,1,n2);
  e2=vector(1,n2);
  D2=vector(1,n2);
  I=matrix(1,n,1,3);
  I2=f3tensor(1,n,1,n2,1,3);
  S=complexvector(1,nt);        for(it1=1;it1<=nt;it1++) {S[it1].re=0;S[it1].im=0;}
  Rr=complexmatrix(1,nt,1,nt);  for(it1=1;it1<=nt;it1++) for(it3=1;it3<=nt;it3++) {Rr[it1][it3].re=0;Rr[it1][it3].im=0;}
  Rnr=complexmatrix(1,nt,1,nt); for(it1=1;it1<=nt;it1++) for(it3=1;it3<=nt;it3++) {Rnr[it1][it3].re=0;Rnr[it1][it3].im=0;}

istruct=0;
do
  {istruct++;

/*read coordinates*/
  n=coordinate_read(x,y,z,protein);
  if(n==0) break;                           /*end of file reached*/
  n2=n*(n+1)/2;

/*Calculate coupling matrix H1 and local mode transition dipoles vxd,vyd,vzd*/ 
  interaction(H1,vxd,vyd,vzd,x,y,z,n,protein);

/*Add inhomogeneous broadening to diagonal, mimicking solvent fluctuations*/
  for (i=1;i<=n;i++) H1[i][i]+=woff+w_inhomogeneous*gasdev(&idum);


/*Print one-excititon Hamiltonian*/  
  printf("\n\nStructure #%d\n",istruct);
  for(i=1;i<=n;i++)
    {
    for(j=1;j<=n;j++) printf(" %5.1f ",H1[i][j]);
    printf("\n");
    }

/*Calculate two-eciton Hamiltonain from 1-exciton Hamiltonian*/
  two_exciton_matrix(H2,H1,n,n2);                         

/*Diagonalize one-exciton Hamiltonian*/
  tred2(H1,n,D,e);
  tqli(D,e,n,H1);
  eigsrt(D,H1,n);
  dipole_one_exciton(I,H1,vxd,vyd,vzd,n);                    

/*Calculate linear response function*/
  LinResponse(S,D,I,n);

/*Diagonalize two-exciton Hamiltonian*/
  tred2(H2,n2,D2,e2);
  tqli(D2,e2,n2,H2);
  eigsrt(D2,H2,n2);
  dipole_two_exciton(I2,H2,H1,vxd,vyd,vzd,n2,n);
  NonlinResponse(Rr,Rnr,D,D2,I,I2,n,n2,0);
  } while(0==0);

/*Calculate 1D-Fouriertrafo of linear response*/
fft(S,nt);

/*Calculate 2D-Fouriertrafo*/
fft2D(Rr,nt);
fft2D(Rnr,nt);

write_spectra(S,Rr,Rnr,protein);

}

/******************************************************************************/
void NonlinResponse(complex **Rr,complex **Rnr,float *D,float *D2,float **I,float ***I2,int n,int n2,float t2)
{
int it1,it3,i,j,k;
float dipole,angle,mui,muj,muik,mujk,cos1,cos2,cos3,cos4,cos5,cos6;
complex prop1,prop3,phase1,phase3,tmp;

/*Calculates nonlinear response functions for <ZZZZ>*/ 
/*Rather than calculating Cosine, Sine and Exponentials at each time-step of the t1,t3 loop, it is claculated only once*/
/*at the very beginning, and then time-propagated; complicates the code a bit, but is signifignificantly faster*/

  for(j=1;j<=n;j++) for(i=1;i<=n;i++) 
    {
/*Rephasing diagrams R1*/
    muj=sqrt(I[j][1]*I[j][1]+I[j][2]*I[j][2]+I[j][3]*I[j][3]);mui=sqrt(I[i][1]*I[i][1]+I[i][2]*I[i][2]+I[i][3]*I[i][3]);
    dipole=mui*mui*muj*muj;
    cos1=(I[i][1]*I[j][1]+I[i][2]*I[j][2]+I[i][3]*I[j][3])/mui/muj;
    angle=(1+2*cos1*cos1)/15;
    prop1.re=cos(+D[j]*cw*dt)*exp(-dt/T2); prop1.im=sin(+D[j]*cw*dt)*exp(-dt/T2);
    prop3.re=cos(-D[i]*cw*dt)*exp(-dt/T2); prop3.im=sin(-D[i]*cw*dt)*exp(-dt/T2);
    phase1.re=-angle*dipole*cos(+(D[j]-D[i])*cw*t2); phase1.im=-angle*dipole*sin(+(D[j]-D[i])*cw*t2);
    for(it1=1;it1<=nt;it1++)
      {
      phase3.re=phase1.re;phase3.im=phase1.im;
      for(it3=1;it3<=nt;it3++)
        {
        Rr[it3][it1].re+=phase3.re;
        Rr[it3][it1].im+=phase3.im;
        tmp.re=phase3.re*prop3.re-phase3.im*prop3.im;tmp.im=phase3.re*prop3.im+phase3.im*prop3.re;
        phase3.re=tmp.re;phase3.im=tmp.im;
        }
      tmp.re=phase1.re*prop1.re-phase1.im*prop1.im;tmp.im=phase1.re*prop1.im+phase1.im*prop1.re;
      phase1.re=tmp.re;phase1.im=tmp.im;
      }

/*Rephasing diagrams R2*/
    prop1.re=cos(+D[j]*cw*dt)*exp(-dt/T2); prop1.im=sin(+D[j]*cw*dt)*exp(-dt/T2);
    prop3.re=cos(-D[i]*cw*dt)*exp(-dt/T2); prop3.im=sin(-D[i]*cw*dt)*exp(-dt/T2);
    phase1.re=-angle*dipole; phase1.im=0;
    for(it1=1;it1<=nt;it1++)
      {
      phase3.re=phase1.re;phase3.im=phase1.im;
      for(it3=1;it3<=nt;it3++)
        {
        Rr[it3][it1].re+=phase3.re;
        Rr[it3][it1].im+=phase3.im;
        tmp.re=phase3.re*prop3.re-phase3.im*prop3.im;tmp.im=phase3.re*prop3.im+phase3.im*prop3.re;
        phase3.re=tmp.re;phase3.im=tmp.im;
        }
      tmp.re=phase1.re*prop1.re-phase1.im*prop1.im;tmp.im=phase1.re*prop1.im+phase1.im*prop1.re;
      phase1.re=tmp.re;phase1.im=tmp.im;
      }

/*Rephasing diagrams R3*/
    for(k=1;k<=n2;k++)
      {
      muj=sqrt(I[j][1]*I[j][1]+I[j][2]*I[j][2]+I[j][3]*I[j][3]);
      mui=sqrt(I[i][1]*I[i][1]+I[i][2]*I[i][2]+I[i][3]*I[i][3]);
      mujk=sqrt(I2[j][k][1]*I2[j][k][1]+I2[j][k][2]*I2[j][k][2]+I2[j][k][3]*I2[j][k][3]);
      muik=sqrt(I2[i][k][1]*I2[i][k][1]+I2[i][k][2]*I2[i][k][2]+I2[i][k][3]*I2[i][k][3]);
      dipole=muj*mui*mujk*muik;
      cos1=(I[i][1]*I[j][1]+I[i][2]*I[j][2]+I[i][3]*I[j][3])/mui/muj;
      cos2=(I2[i][k][1]*I2[j][k][1]+I2[i][k][2]*I2[j][k][2]+I2[i][k][3]*I2[j][k][3])/muik/mujk;
      cos3=(I[i][1]*I2[i][k][1]+I[i][2]*I2[i][k][2]+I[i][3]*I2[i][k][3])/mui/muik;
      cos4=(I[j][1]*I2[j][k][1]+I[j][2]*I2[j][k][2]+I[j][3]*I2[j][k][3])/muj/mujk;
      cos5=(I[i][1]*I2[j][k][1]+I[i][2]*I2[j][k][2]+I[i][3]*I2[j][k][3])/mui/mujk;
      cos6=(I[j][1]*I2[i][k][1]+I[j][2]*I2[i][k][2]+I[j][3]*I2[i][k][3])/muj/muik;
      angle=2*(cos1*cos2+cos3*cos4+cos5*cos6)/30;
      prop1.re=cos(+D[j]*cw*dt)*exp(-dt/T2); prop1.im=sin(+D[j]*cw*dt)*exp(-dt/T2);
      prop3.re=cos(-(D2[k]-D[j])*cw*dt)*exp(-dt/T2); prop3.im=sin(-(D2[k]-D[j])*cw*dt)*exp(-dt/T2);
      phase1.re=+angle*dipole*cos(+(D[j]-D[i])*cw*t2); phase1.im=+angle*dipole*sin(+(D[j]-D[i])*cw*t2);
      for(it1=1;it1<=nt;it1++)
        {
        phase3.re=phase1.re;phase3.im=phase1.im;
        for(it3=1;it3<=nt;it3++)
          {
          Rr[it3][it1].re+=phase3.re;
          Rr[it3][it1].im+=phase3.im;
          tmp.re=phase3.re*prop3.re-phase3.im*prop3.im;tmp.im=phase3.re*prop3.im+phase3.im*prop3.re;
          phase3.re=tmp.re;phase3.im=tmp.im;
          }
        tmp.re=phase1.re*prop1.re-phase1.im*prop1.im;tmp.im=phase1.re*prop1.im+phase1.im*prop1.re;
        phase1.re=tmp.re;phase1.im=tmp.im;
        }
      }
    }


  for(j=1;j<=n;j++) for(i=1;i<=n;i++)     
    {
/*Non-rephasing diagrams R4*/
    muj=sqrt(I[j][1]*I[j][1]+I[j][2]*I[j][2]+I[j][3]*I[j][3]);mui=sqrt(I[i][1]*I[i][1]+I[i][2]*I[i][2]+I[i][3]*I[i][3]);
    dipole=mui*mui*muj*muj;
    cos1=(I[i][1]*I[j][1]+I[i][2]*I[j][2]+I[i][3]*I[j][3])/mui/muj;
    angle=(1+2*cos1*cos1)/15;
    prop1.re=cos(-D[j]*cw*dt)*exp(-dt/T2); prop1.im=sin(-D[j]*cw*dt)*exp(-dt/T2);
    prop3.re=cos(-D[j]*cw*dt)*exp(-dt/T2); prop3.im=sin(-D[j]*cw*dt)*exp(-dt/T2);
    phase1.re=-angle*dipole*cos(-(D[j]-D[i])*cw*t2); phase1.im=-angle*dipole*sin(-(D[j]-D[i])*cw*t2);
    for(it1=1;it1<=nt;it1++)
      {
      phase3.re=phase1.re;phase3.im=phase1.im;
      for(it3=1;it3<=nt;it3++)
        {
        Rnr[it3][it1].re+=phase3.re;
        Rnr[it3][it1].im+=phase3.im;
        tmp.re=phase3.re*prop3.re-phase3.im*prop3.im;tmp.im=phase3.re*prop3.im+phase3.im*prop3.re;
        phase3.re=tmp.re;phase3.im=tmp.im;
        }
      tmp.re=phase1.re*prop1.re-phase1.im*prop1.im;tmp.im=phase1.re*prop1.im+phase1.im*prop1.re;
      phase1.re=tmp.re;phase1.im=tmp.im;
      }

/*Non-rephasing diagrams R5*/
    prop1.re=cos(-D[j]*cw*dt)*exp(-dt/T2); prop1.im=sin(-D[j]*cw*dt)*exp(-dt/T2);
    prop3.re=cos(-D[i]*cw*dt)*exp(-dt/T2); prop3.im=sin(-D[i]*cw*dt)*exp(-dt/T2);
    phase1.re=-angle*dipole; phase1.im=0;
    for(it1=1;it1<=nt;it1++)
      {
      phase3.re=phase1.re;phase3.im=phase1.im;
      for(it3=1;it3<=nt;it3++)
        {
        Rnr[it3][it1].re+=phase3.re;
        Rnr[it3][it1].im+=phase3.im;
        tmp.re=phase3.re*prop3.re-phase3.im*prop3.im;tmp.im=phase3.re*prop3.im+phase3.im*prop3.re;
        phase3.re=tmp.re;phase3.im=tmp.im;
        }
      tmp.re=phase1.re*prop1.re-phase1.im*prop1.im;tmp.im=phase1.re*prop1.im+phase1.im*prop1.re;
      phase1.re=tmp.re;phase1.im=tmp.im;
      }

/*Non-rephasing diagrams R6*/
    for(k=1;k<=n2;k++)
      {
      muj=sqrt(I[j][1]*I[j][1]+I[j][2]*I[j][2]+I[j][3]*I[j][3]);
      mui=sqrt(I[i][1]*I[i][1]+I[i][2]*I[i][2]+I[i][3]*I[i][3]);
      mujk=sqrt(I2[j][k][1]*I2[j][k][1]+I2[j][k][2]*I2[j][k][2]+I2[j][k][3]*I2[j][k][3]);
      muik=sqrt(I2[i][k][1]*I2[i][k][1]+I2[i][k][2]*I2[i][k][2]+I2[i][k][3]*I2[i][k][3]);
      dipole=muj*mui*mujk*muik;
      cos1=(I[i][1]*I[j][1]+I[i][2]*I[j][2]+I[i][3]*I[j][3])/mui/muj;
      cos2=(I2[i][k][1]*I2[j][k][1]+I2[i][k][2]*I2[j][k][2]+I2[i][k][3]*I2[j][k][3])/muik/mujk;
      cos3=(I[i][1]*I2[i][k][1]+I[i][2]*I2[i][k][2]+I[i][3]*I2[i][k][3])/mui/muik;
      cos4=(I[j][1]*I2[j][k][1]+I[j][2]*I2[j][k][2]+I[j][3]*I2[j][k][3])/muj/mujk;
      cos5=(I[i][1]*I2[j][k][1]+I[i][2]*I2[j][k][2]+I[i][3]*I2[j][k][3])/mui/mujk;
      cos6=(I[j][1]*I2[i][k][1]+I[j][2]*I2[i][k][2]+I[j][3]*I2[i][k][3])/muj/muik;
      angle=2*(cos1*cos2+cos3*cos4+cos5*cos6)/30;
      prop1.re=cos(-D[j]*cw*dt)*exp(-dt/T2); prop1.im=sin(-D[j]*cw*dt)*exp(-dt/T2);
      prop3.re=cos(-(D2[k]-D[i])*cw*dt)*exp(-dt/T2); prop3.im=sin(-(D2[k]-D[i])*cw*dt)*exp(-dt/T2);
      phase1.re=+angle*dipole*cos(-(D[j]-D[i])*cw*t2); phase1.im=+angle*dipole*sin(-(D[j]-D[i])*cw*t2);
      for(it1=1;it1<=nt;it1++)
        {
        phase3.re=phase1.re;phase3.im=phase1.im;
        for(it3=1;it3<=nt;it3++)
          {
          Rnr[it3][it1].re+=phase3.re;
          Rnr[it3][it1].im+=phase3.im;
          tmp.re=phase3.re*prop3.re-phase3.im*prop3.im;tmp.im=phase3.re*prop3.im+phase3.im*prop3.re;
          phase3.re=tmp.re;phase3.im=tmp.im;
          }
        tmp.re=phase1.re*prop1.re-phase1.im*prop1.im;tmp.im=phase1.re*prop1.im+phase1.im*prop1.re;
        phase1.re=tmp.re;phase1.im=tmp.im;
        }
      }
    }

}

/******************************************************************************/
void LinResponse(complex *S,float *D,float **I, int n)
{
int it1,i;
float dipole;

  for(i=1;i<=n;i++) for(it1=1;it1<=nt;it1++) 
    {
    dipole=I[i][1]*I[i][1]+I[i][2]*I[i][2]+I[i][3]*I[i][3];
    S[it1].re+=dipole*cos(-D[i]*cw*(it1-1)*dt)*exp(-(it1-1)*dt/T2);
    S[it1].im+=dipole*sin(-D[i]*cw*(it1-1)*dt)*exp(-(it1-1)*dt/T2);
    }
}


/******************************************************************************/
void interaction(float **H1,float *vxd,float *vyd,float *vzd,float **x,float **y,float **z,int n,char *name)
{
int i,j;

for(i=1;i<=n;i++) for(j=1;j<=n;j++) H1[i][j]=0;
 
transition_charges(H1,vxd,vyd,vzd,x,y,z,n);       /*Use transition charges from ab-initio calculations; this also calculets dipoles*/

parametrizedB3LYP(H1,x,y,z,n);                    /*Use a pre-parametrized B3LYP calculation for nearest neigbors*/ 
  
Hydrogenbond(H1,x,y,z,n);                         /*Search for hydrogen bonds and calculate frequency shifts*/  
}

/********************************************************************************************/
#define h 6.62607e-34
#define amu 1.6605402e-27
#define c 2.9979e10
#define Debey 4.803
#define e0 8.854e-12
#define e  1.602e-19

void transition_charges(float **H1,float *vxd,float *vyd,float *vzd,float **x,float **y,float **z,int n)
{
float xmode[5],ymode[5],zmode[5],qmode[5],Freq,Mass;
float x_der[5],y_der[5],z_der[5],q_der[5];
float vxCOrev,vyCOrev,vzCOrev,vxCO,vyCO,vzCO;
float vxCNrev,vyCNrev,vzCNrev,vxCN,vyCN,vzCN;
float aCO,aCN,Amplitude,xtmp,q,sig,q1,q2,q3,q4,r1,r2,r3,r4;
float ***xd, ***yd, ***zd;
FILE *file;
int i,i1,i2,j,j1,j2,l,sign[nmax];

xd=f3tensor(1,2,1,n,1,4);yd=f3tensor(1,2,1,n,1,4);zd=f3tensor(1,2,1,n,1,4);

/*Load parameter files for displacements, charges and charge flows*/
if((file=fopen("nma.par","rt"))==NULL) {printf ("\nCan't open %s\n","nma.par");exit(1);}
fscanf(file,"%f ",&Freq);
fscanf(file,"%f ",&Mass);
for (j=1;j<=4;j++) fscanf(file,"%f %f %f %f",&xmode[j],&ymode[j],&zmode[j],&qmode[j]);
for (j=1;j<=4;j++) fscanf(file,"%f %f %f %f",&x_der[j],&y_der[j],&z_der[j],&q_der[j]);

      
/*Calculate displaced atoms*/
Amplitude=1e10*sqrt(h/(8*PI*PI*Freq*c*Mass*amu));
for (i=1;i<=n;i++)
  {
  vxCOrev=xmode[1]-xmode[2];vyCOrev=ymode[1]-ymode[2];vzCOrev=zmode[1]-zmode[2];
  vxCNrev=xmode[1]-xmode[3];vyCNrev=ymode[1]-ymode[3];vzCNrev=zmode[1]-zmode[3];
  vxCO=x[i][1]-x[i][2];vyCO=y[i][1]-y[i][2];vzCO=z[i][1]-z[i][2];
  vxCN=x[i][1]-x[i][3];vyCN=y[i][1]-y[i][3];vzCN=z[i][1]-z[i][3];

  for (j=1;j<=4;j++)
    {
    aCO=(x_der[j]*vxCOrev+y_der[j]*vyCOrev+z_der[j]*vzCOrev)/length(vxCOrev,vyCOrev,vzCOrev)/length(vxCO,vyCO,vzCO);
    aCN=(x_der[j]*vxCNrev+y_der[j]*vyCNrev+z_der[j]*vzCNrev)/length(vxCNrev,vyCNrev,vzCNrev)/length(vxCN,vyCN,vzCN);
    if (j==1)
      {
      sign[i]=1;                                /*Test sign of Eigenmode by movement of C-atom)*/
      if (aCO>0) sign[i]=-1;
      }

    xd[1][i][j]=x[i][j]-sign[i]*Amplitude/2.*(aCO*vxCO+aCN*vxCN);
    yd[1][i][j]=y[i][j]-sign[i]*Amplitude/2.*(aCO*vyCO+aCN*vyCN);
    zd[1][i][j]=z[i][j]-sign[i]*Amplitude/2.*(aCO*vzCO+aCN*vzCN);
    xd[2][i][j]=x[i][j]+sign[i]*Amplitude/2.*(aCO*vxCO+aCN*vxCN);
    yd[2][i][j]=y[i][j]+sign[i]*Amplitude/2.*(aCO*vyCO+aCN*vyCN);
    zd[2][i][j]=z[i][j]+sign[i]*Amplitude/2.*(aCO*vzCO+aCN*vzCN);
    }
  }

/*first calculate transition dipoles*/
for (i=1;i<=n;i++)
  {
  vxd[i]=0;vyd[i]=0;vzd[i]=0;
  for (j=1;j<=4;j++)
    for (l=1,sig=-1;l<=2;l++,sig+=2)
      {
      q=qmode[j]+sign[i]*0.5*sig*q_der[j];
      vxd[i]+=sig*xd[l][i][j]*q*Debey;
      vyd[i]+=sig*yd[l][i][j]*q*Debey;
      vzd[i]+=sig*zd[l][i][j]*q*Debey;
      }
  /*printf("%f\n",length(vxd[i],vyd[i],vzd[i]));*/
  }

/*calculate interaction matrix*/
for (i1=2;i1<=n;i1++)
  for (i2=1;i2<i1;i2++)
    {
    H1[i1][i2]=0;
    for(j1=1;j1<=4;j1++)
      for(j2=1;j2<=4;j2++)
        {
        q1=(qmode[j1]-0.5*sign[i1]*q_der[j1])*(qmode[j2]-0.5*sign[i2]*q_der[j2]);
        q2=(qmode[j1]+0.5*sign[i1]*q_der[j1])*(qmode[j2]-0.5*sign[i2]*q_der[j2]);
        q3=(qmode[j1]-0.5*sign[i1]*q_der[j1])*(qmode[j2]+0.5*sign[i2]*q_der[j2]);
        q4=(qmode[j1]+0.5*sign[i1]*q_der[j1])*(qmode[j2]+0.5*sign[i2]*q_der[j2]);
        r1=length(xd[1][i1][j1]-xd[1][i2][j2],yd[1][i1][j1]-yd[1][i2][j2],zd[1][i1][j1]-zd[1][i2][j2]);
        r2=length(xd[2][i1][j1]-xd[1][i2][j2],yd[2][i1][j1]-yd[1][i2][j2],zd[2][i1][j1]-zd[1][i2][j2]);
        r3=length(xd[1][i1][j1]-xd[2][i2][j2],yd[1][i1][j1]-yd[2][i2][j2],zd[1][i1][j1]-zd[2][i2][j2]);
        r4=length(xd[2][i1][j1]-xd[2][i2][j2],yd[2][i1][j1]-yd[2][i2][j2],zd[2][i1][j1]-zd[2][i2][j2]);
        H1[i1][i2]+=(q1/r1+q4/r4-q2/r2-q3/r3);
        }
    H1[i1][i2]=e*e/4/PI/e0*1e10/h/c*H1[i1][i2];
    H1[i2][i1]=H1[i1][i2];
    }
    
free_f3tensor(xd,1,2,1,n,1,4);free_f3tensor(yd,1,2,1,n,1,4);free_f3tensor(zd,1,2,1,n,1,4);
}

#undef h 
#undef amu
#undef c
#undef e
#undef e0
#undef Debey


/********************************************************************************************/
void parametrizedB3LYP(float **H1,float **x,float **y,float **z,int n)
{
int i,j,k;
float phi,psi,para[75],coupling;
FILE *file;

/*reads precalculated coupling for nearest neighbors, stored as Fourier series*/

if((file=fopen("coupling.par","rt"))==NULL) {printf ("\nCan't open %s\n","coupling.par");exit(1);}
for(i=1;i<=74;i++) fscanf(file,"%f ",&para[i]);
fclose(file);

for(i=1;i<=n-1;i++)
  {
  phi=dehydral(x[i][1],x[i][3],x[i][5],x[i+1][1],y[i][1],y[i][3],y[i][5],y[i+1][1],z[i][1],z[i][3],z[i][5],z[i+1][1]);
  psi=dehydral(x[i][3],x[i][5],x[i+1][1],x[i+1][3],y[i][3],y[i][5],y[i+1][1],y[i+1][3],z[i][3],z[i][5],z[i+1][1],z[i+1][3]);
  coupling=0;
  for(k=0;k<=6;k++) for(j=0;j<=6;j++) coupling+=para[j+1+k*7]*cos(j*psi/180.*PI)*cos(k*phi/180.*PI);
  for(k=1;k<=5;k++) for(j=1;j<=5;j++) coupling+=para[j+49+(k-1)*5]*sin(j*psi/180.*PI)*sin(k*phi/180.*PI);
  H1[i][i+1]=coupling;
  H1[i+1][i]=coupling;
  }
}
/*********************************************************************************/
void Hydrogenbond(float **H1,float **x,float **y,float **z,int n)

#define anglehydrogen     60          /*threshold hydrogenbond angle*/

{
int i,j,j1;
float dist1,dist2,alpha;

for (j=1;j<=n;j++)
  {
  for (i=1;i<=n;i++)
    {
    dist1=sqrt((x[i][4]-x[j][2])*(x[i][4]-x[j][2])+(y[i][4]-y[j][2])*(y[i][4]-y[j][2])+(z[i][4]-z[j][2])*(z[i][4]-z[j][2]));
    dist2=sqrt((x[i][4]-x[i][3])*(x[i][4]-x[i][3])+(y[i][4]-y[i][3])*(y[i][4]-y[i][3])+(z[i][4]-z[i][3])*(z[i][4]-z[i][3]));
    alpha=((x[i][3]-x[i][4])*(x[j][2]-x[i][4])+(y[i][3]-y[i][4])*(y[j][2]-y[i][4])+(z[i][3]-z[i][4])*(z[j][2]-z[i][4]))/dist1/dist2;
    

    if(dist1<=2.6&&alpha<-cos(anglehydrogen/180.*PI))
      {
      H1[j][j]-=hydrogenbondshift*(2.6-dist1);                                            
      //printf("Hydrogenbond: %2d  %2d  Distance: %5.3f  Angle: %5.3f \n",i,j,dist1,alpha);
      }
     }
  }

}
/*******************************************************************************************/
float dehydral (float x1,float x2,float x3,float x4, float y1,float y2,float y3,float y4, float z1,float z2,float z3,float z4)
{
float vx21,vy21,vz21,vx23,vy23,vz23,vx34,vy34,vz34,tmp,angle,vx,vy,vz;

vx21=x1-x2;vy21=y1-y2;vz21=z1-z2;
vx23=x3-x2;vy23=y3-y2;vz23=z3-z2;
vx34=x4-x3;vy34=y4-y3;vz34=z4-z3;

tmp=(vx21*vx23+vy21*vy23+vz21*vz23)/(vx23*vx23+vy23*vy23+vz23*vz23);
vx21=vx21-tmp*vx23;vy21=vy21-tmp*vy23;vz21=vz21-tmp*vz23;
tmp=(vx34*vx23+vy34*vy23+vz34*vz23)/(vx23*vx23+vy23*vy23+vz23*vz23);
vx34=vx34-tmp*vx23;vy34=vy34-tmp*vy23;vz34=vz34-tmp*vz23;

angle=(vx21*vx34+vy21*vy34+vz21*vz34)/length(vx21,vy21,vz21)/length(vx34,vy34,vz34);
angle=180/PI*acos(angle);
vx=vy21*vz34-vz21*vy34;vy=vz21*vx34-vx21*vz34;vz=vx21*vy34-vy21*vx34;

if ((vx*vx23+vy*vy23+vz*vz23)<0) angle=-angle;
  
/*printf("%f \n",angle);*/
return(angle);
}

/*********************************************************************************/
void dipole_one_exciton(float **I,float **Q,float *vxd,float *vyd,float *vzd,int n)
{
int i,j;
float a1,a2,a3;

for (i=1;i<=n;i++)
  {
  a1=0; a2=0; a3=0;
  for (j=1;j<=n;j++)
    {
      a1+=Q[j][i]*vxd[j]; a2+=Q[j][i]*vyd[j]; a3+=Q[j][i]*vzd[j];
    }
  I[i][1]=a1;
  I[i][2]=a2;
  I[i][3]=a3;
  }
}
/********************************************************************************/
void dipole_two_exciton(float ***I2,float **Q2,float **Q,float *vxd,float *vyd,float *vzd,int n2,int n)
{
int m1,m2,i,j,k,jk;
float a1,a2,a3;

for (m1=1;m1<=n;m1++)
  for (m2=1;m2<=n2;m2++)
    {
    a1=0; a2=0; a3=0;
    for (i=1;i<=n;i++)
      {
      jk=0;
      for (j=1;j<=n;j++)
        for (k=j;k<=n;k++)
          {
          jk++;
          if ((j==k)&&(j==i))
	    {a1+=sqrt(2)*vxd[i]*Q[i][m1]*Q2[jk][m2];
              a2+=sqrt(2)*vyd[i]*Q[i][m1]*Q2[jk][m2];
              a3+=sqrt(2)*vzd[i]*Q[i][m1]*Q2[jk][m2];}
          if ((j!=k)&&(i==k))
	    {a1+=vxd[j]*Q[i][m1]*Q2[jk][m2];
              a2+=vyd[j]*Q[i][m1]*Q2[jk][m2];
              a3+=vzd[j]*Q[i][m1]*Q2[jk][m2];}
          if ((j!=k)&&(i==j))
	    {a1+=vxd[k]*Q[i][m1]*Q2[jk][m2];
              a2+=vyd[k]*Q[i][m1]*Q2[jk][m2];
              a3+=vzd[k]*Q[i][m1]*Q2[jk][m2];}
          }
      }
    I2[m1][m2][1]=a1;
    I2[m1][m2][2]=a2;
    I2[m1][m2][3]=a3;
    }
}
/**********************************************************************************/
void two_exciton_matrix(float **H2,float **H1,int n,int n2)
{
int i,j,k,l,i1,i2;

i1=0;i2=0;

for (i=1;i<=n;i++)
  for (j=i;j<=n;j++)
    {
    i1++;
    i2=0;
    for (k=1;k<=n;k++)
     for (l=k;l<=n;l++)
       {
       i2++;
       H2[i1][i2]=0;
       if ((i==k)&&(j==l)&&(i==j)) H2[i1][i2]=2*H1[i][i]-anharmonicity;    /* <2i|V|2i> */

       if ((i==k)&&(j==l)&&(i!=j)) H2[i1][i2]=H1[i][i]+H1[j][j];           /* <1i1j|V|1i1j> */

       if ((i==j)&&(j==l)&&(i!=k)) H2[i1][i2]=sqrt(2)*H1[i][k];            /* <2i|V|1i1j>  */
       if ((i==j)&&(i==k)&&(j!=l)) H2[i1][i2]=sqrt(2)*H1[i][l];
       if ((k==l)&&(i==k)&&(j!=l)) H2[i1][i2]=sqrt(2)*H1[j][l];
       if ((k==l)&&(j==l)&&(i!=k)) H2[i1][i2]=sqrt(2)*H1[i][k];

       if ((i==k)&&(j!=l)&&(i!=j)&&(k!=l)) H2[i1][i2]=H1[j][l];             /* <ij|V|kl>  */
       if ((j==l)&&(i!=k)&&(i!=j)&&(k!=l)) H2[i1][i2]=H1[i][k];
       if ((i==l)&&(j!=k)&&(i!=j)&&(k!=l)) H2[i1][i2]=H1[j][k];
       if ((j==k)&&(i!=l)&&(i!=j)&&(k!=l)) H2[i1][i2]=H1[i][l];
       }
    }
}
/***********************************************************************************/
int coordinate_read(float **x, float **y, float **z, char *protein)
{
int i,istructure=0,n;
char infile[100], inpuff[100], term_string[4]="END",atom_string[5]="ATOM",d[30];
static FILE *file;
static int firstmark=0;

if(firstmark==0)
  {
  strcpy(infile,protein);
  strcat(infile,".pdb");
  if((file=fopen(infile,"rt"))==NULL) {printf ("\nCan't open file %s\n",infile);exit(1);}
  firstmark=1;
  }

i=0;
do {
   if (fgets(inpuff,100,file)==NULL) {fclose(file);return(0);}
   if (strncmp(inpuff,atom_string,4)==0)
     {
     if((inpuff[13]=='H' && inpuff[14]==' ')||(inpuff[13]=='C' && inpuff[14]=='N'))
       {
       sscanf(&inpuff[23],"%d",&i);
       sscanf(&inpuff[23],"%s %f %f %f",&d,&x[i][4],&y[i][4],&z[i][4]);
//       printf("H: %3i %8.3f %8.3f %8.3f\n",i,x[i][4],y[i][4],z[i][4]);
       }
     if(inpuff[13]=='C' && inpuff[14]==' ')
       {
       sscanf(&inpuff[23],"%d",&i);
       sscanf(&inpuff[23],"%s %f %f %f",&d,&x[i][1],&y[i][1],&z[i][1]);
//       printf("C: %3i %8.3f %8.3f %8.3f\n",i,x[i][1],y[i][1],z[i][1]);
       }
     if(inpuff[13]=='O' && inpuff[14]==' ')
       {
       sscanf(&inpuff[23],"%d",&i);
       sscanf(&inpuff[23],"%s %f %f %f",&d,&x[i][2],&y[i][2],&z[i][2]);
//       printf("O: %3i %8.3f %8.3f %8.3f\n",i,x[i][2],y[i][2],z[i][2]);
       }
     if(inpuff[13]=='N' && inpuff[14]==' ')
       {
       sscanf(&inpuff[23],"%d",&i);
       sscanf(&inpuff[23],"%s %f %f %f",&d,&x[i][3],&y[i][3],&z[i][3]);
//       printf("N: %3i %8.3f %8.3f %8.3f\n",i,x[i][3],y[i][3],z[i][3]);
       }
       
     if(inpuff[13]=='C' && inpuff[14]=='A' &&inpuff[15]==' ')
       {
       sscanf(&inpuff[23],"%d",&i);
       sscanf(&inpuff[23],"%s %f %f %f",&d,&x[i][5],&y[i][5],&z[i][5]); 
//       printf("CA:%3i %8.3f %8.3f %8.3f\n",i,x[i][5],y[i][5],z[i][5]);
       }    

     }
  }  while (strncmp(inpuff,term_string,3)!=0);

if(i==0) return(0); /*end of file reached*/

/*resort labelling with respect to amino acids into one with respect to peptide units*/
  n=i-1;
  for (i=2;i<=n+1;i++)
    {
    x[i-1][3]=x[i][3];y[i-1][3]=y[i][3];z[i-1][3]=z[i][3];
    x[i-1][4]=x[i][4];y[i-1][4]=y[i][4];z[i-1][4]=z[i][4];
    x[i-1][5]=x[i][5];y[i-1][5]=y[i][5];z[i-1][5]=z[i][5];
//    printf("%f\n",length(x[i-1][3]-x[i-1][4],y[i-1][3]-y[i-1][4],z[i-1][3]-z[i-1][4]));
    }
    
return(n);
}

/*********************************************************************************/
void fft(complex *P, int n)
/*prepares data for Fouriertransform; i.e. sorts them to be compatibel with Numerical Reciepies routine, 
takes care of 1/2 factor for first data point, and sorts the output data such the zero frequency is in the middle*/
{
int i,j,ir;
float *a;

  P[1].re/=2;
  P[1].im/=2;

  a=vector(1,2*n);
  for (i=1;i<=n;i++)
  {
      a[2*(i-1)+1]=P[i].re;
      a[2*(i-1)+2]=P[i].im;
  }
  four1(a,n,+1);
  for (i=1;i<=n;i++)
  {
      ir=i+n/2;
      if(ir>n)ir-=n;
      P[i].re=a[2*ir-1]/sqrt(n);
      P[i].im=a[2*ir]/sqrt(n);
  }
free_vector(a,1,2*n);
}

/*******************************************************************************/
void fft2D(complex **R, int n)
{
int it1,it3;
complex *tmp;

tmp=complexvector(1,n);

for(it3=1;it3<=n;it3++) fft(R[it3],n);

for(it1=1;it1<=n;it1++)
  {
  for(it3=1;it3<=n;it3++) {tmp[it3].re=R[it3][it1].re; tmp[it3].im=R[it3][it1].im;}
  fft(tmp,n);
  for(it3=1;it3<=n;it3++) {R[it3][it1].re=tmp[it3].re; R[it3][it1].im=tmp[it3].im;}
  }
free_complexvector(tmp,1,n);
}

/*******************************************************************************/

void write_spectra(complex *S,complex **Rr,complex **Rnr,char *protein)
{
int i1,i3,j;
char outfile[15];
FILE *out;

    strcpy(outfile,protein);                                /* Print linear Spectrum */
    strcat(outfile,"_lin.dat");
    out=fopen(outfile,"wt");
    for (i1=2;i1<=nt;i1++)
      fprintf(out,"%6.1f %8.4f \n",-woff+w0+(i1-1-nt/2)*2*PI/nt/dt/cw,S[i1].re);
    fclose (out);

    strcpy(outfile,protein);                                /* Print linear Spectrum */
    strcat(outfile,"_2D.dat");
    out=fopen(outfile,"wt");
    for(i1=2;i1<=nt;i1++) for(i3=2;i3<=nt;i3++) 
      fprintf(out,"%6.1f %6.1f %8.4f\n",-woff+w0+(i1-1-nt/2)*2*PI/nt/dt/cw,-woff+w0+(i3-1-nt/2)*2*PI/nt/dt/cw,
                         (Rr[i3][nt+2-i1].re+Rnr[i3][i1].re));
    fclose(out);
}


/********************************************************************************/

float length(float x,float y, float z)
    {
    float len;

    len=sqrt(x*x+y*y+z*z);
    return (len);
    }

/***********************************************************************************/
/*                       Numerical Recipis Routines                                */
/***********************************************************************************/

#define SWAP(a,b) tempr=(a);(a)=(b);(b)=tempr

void four1(float data[], unsigned long nn, int isign)
{
        unsigned long n,mmax,m,j,istep,i;
        double wtemp,wr,wpr,wpi,wi,theta;
        float tempr,tempi;

        n=nn << 1;
        j=1;
        for (i=1;i<n;i+=2) {
                if (j > i) {
                        SWAP(data[j],data[i]);
                        SWAP(data[j+1],data[i+1]);
                }
                m=n >> 1;
                while (m >= 2 && j > m) {
                        j -= m;
                        m >>= 1;
                }
                j += m;
        }
        mmax=2;
        while (n > mmax) {
                istep=mmax << 1;
                theta=isign*(6.28318530717959/mmax);
                wtemp=sin(0.5*theta);
                wpr = -2.0*wtemp*wtemp;
                wpi=sin(theta);
                wr=1.0;
                wi=0.0;
                for (m=1;m<mmax;m+=2) {
                        for (i=m;i<=n;i+=istep) {
                                j=i+mmax;
                                tempr=wr*data[j]-wi*data[j+1];
                                tempi=wr*data[j+1]+wi*data[j];
                                data[j]=data[i]-tempr;
                                data[j+1]=data[i+1]-tempi;
                                data[i] += tempr;
                                data[i+1] += tempi;
                        }
                        wr=(wtemp=wr)*wpr-wi*wpi+wr;
                        wi=wi*wpr+wtemp*wpi+wi;
                }
                mmax=istep;
        }
}
#undef SWAP
/* (C) Copr. 1986-92 Numerical Recipes Software ,2:. */

/*********************************************************************/




#define NRANSI
static float sqrarg;
#define SQR(a) ((sqrarg=(a)) == 0.0 ? 0.0 : sqrarg*sqrarg)
#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))

void tqli(float d[], float e[], int n, float **z)
{
        float pythag(float a, float b);
        int m,l,iter,i,k;
        float s,r,p,g,f,dd,c,b;

        for (i=2;i<=n;i++) e[i-1]=e[i];
        e[n]=0.0;
        for (l=1;l<=n;l++) {
                iter=0;
                do {
		  for (m=l;m<=n-1;m++) {
                                dd=fabs(d[m])+fabs(d[m+1]);
                                if ((float)(fabs(e[m])+dd) == dd) break;
		  }
		  if (m != l) {
                                if (iter++ == 30) nrerror("Too many iterations in tqli");
                                g=(d[l+1]-d[l])/(2.0*e[l]);
                                r=pythag(g,1.0);
                                g=d[m]-d[l]+e[l]/(g+SIGN(r,g));
                                s=c=1.0;
                                p=0.0;
                                for (i=m-1;i>=l;i--) {
                                        f=s*e[i];
                                        b=c*e[i];
                                        e[i+1]=(r=pythag(f,g));
                                        if (r == 0.0) {
                                                d[i+1] -= p;
                                                e[m]=0.0;
                                                break;
                                        }
                                        s=f/r;
                                        c=g/r;
                                        g=d[i+1]-p;
                                        r=(d[i]-g)*s+2.0*c*b;
                                        d[i+1]=g+(p=s*r);
                                        g=c*r-b;
                                        for (k=1;k<=n;k++) {
                                                f=z[k][i+1];
                                                z[k][i+1]=s*z[k][i]+c*f;
                                                z[k][i]=c*z[k][i]-s*f;
                                        }
                                }
                                if (r == 0.0 && i >= l) continue;
                                d[l] -= p;
                                e[l]=g;
                                e[m]=0.0;
		  }
                } while (m != l);
        }
}

/********************************************************************/

float pythag(float a, float b)
{
        float absa,absb;
        absa=fabs(a);
        absb=fabs(b);
        if (absa > absb) return absa*sqrt(1.0+SQR(absb/absa));
        else return (absb == 0.0 ? 0.0 : absb*sqrt(1.0+SQR(absa/absb)));
}

/*********************************************************************/
void tred2(float **a, int n, float d[], float e[])
{
        int l,k,j,i;
        float scale,hh,h,g,f;

        for (i=n;i>=2;i--) {
                l=i-1;
                h=scale=0.0;
                if (l > 1) {
                        for (k=1;k<=l;k++)
                                scale += fabs(a[i][k]);
                        if (scale == 0.0)
                                e[i]=a[i][l];
                        else {
			  for (k=1;k<=l;k++) {
                                        a[i][k] /= scale;
                                        h += a[i][k]*a[i][k];
			  }
                                f=a[i][l];
                                g=(f >= 0.0 ? -sqrt(h) : sqrt(h));
                                e[i]=scale*g;
                                h -= f*g;
                                a[i][l]=f-g;
                                f=0.0;
                                for (j=1;j<=l;j++) {
                                        a[j][i]=a[i][j]/h;
                                        g=0.0;
                                        for (k=1;k<=j;k++)
                                                g += a[j][k]*a[i][k];
                                        for (k=j+1;k<=l;k++)
                                                g += a[k][j]*a[i][k];
                                        e[j]=g/h;
                                        f += e[j]*a[i][j];
                                }
                                hh=f/(h+h);
                                for (j=1;j<=l;j++) {
                                        f=a[i][j];
                                        e[j]=g=e[j]-hh*f;
                                        for (k=1;k<=j;k++)
                                                a[j][k] -= (f*e[k]+g*a[i][k]);
                                }
                        }
                } else
                        e[i]=a[i][l];
                d[i]=h;
        }
        d[1]=0.0;
        e[1]=0.0;
        /* Contents of this loop can be omitted if eigenvectors not
                        wanted except for statement d[i]=a[i][i]; */
        for (i=1;i<=n;i++) {
                l=i-1;
                if (d[i]) {
		  for (j=1;j<=l;j++) {
                                g=0.0;
                                for (k=1;k<=l;k++)
                                        g += a[i][k]*a[k][j];
                                for (k=1;k<=l;k++)
                                        a[k][j] -= g*a[k][i];
		  }
                }
                d[i]=a[i][i];
                a[i][i]=1.0;
                for (j=1;j<=l;j++) a[j][i]=a[i][j]=0.0;
        }
}







/* (C) Copr. 1986-92 Numerical Recipes Software ,2:. */

/***********************************************************************************/

void eigsrt(float d[], float **v, int n)
{
	int k,j,i;
	float p;

	for (i=1;i<n;i++) {
		p=d[k=i];
		for (j=i+1;j<=n;j++)
			if (d[j] >= p) p=d[k=j];
		if (k != i) {
			d[k]=d[i];
			d[i]=p;
			for (j=1;j<=n;j++) {
				p=v[j][i];
				v[j][i]=v[j][k];
				v[j][k]=p;
			}
		}
	}
}
/* (C) Copr. 1986-92 Numerical Recipes Software ,2:. */

/*************************************************************************************/
float gasdev(long *idum)
{
float ran1(long *idum);
static int iset=0;
static float gset;
float fac,rsq,v1,v2;

	          if  (iset == 0) {
		                    do {
				    v1=2.0*ran1(idum)-1.0;
	                            v2=2.0*ran1(idum)-1.0;
	                            rsq=v1*v1+v2*v2;
				        } while (rsq >= 1.0 || rsq == 0.0);
                                    fac=sqrt(-2.0*log(rsq)/rsq);
 		                    gset=v1*fac;
		                    iset=1;
			            return v2*fac;
		    } else {
			                 iset=0;
	       	                         return gset;
	            }
}

/************************************************************************************/
/******************************************/
#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define MASK 123459876

float ran0(long *idum)
{
        long k;
        float ans;

        *idum ^= MASK;
        k=(*idum)/IQ;
        *idum=IA*(*idum-k*IQ)-IR*k;
        if (*idum < 0) *idum += IM;
        ans=AM*(*idum);
        *idum ^= MASK;
        return ans;
}
#undef IA
#undef IM
#undef AM
#undef IQ
#undef IR
#undef MASK
/****************************************/

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

float ran1(long *idum)
{
          int j;
	  long k;
	  static long iy=0;
	  static long iv[NTAB];
	  float temp;

	  if (*idum <= 0 || !iy) {
		  if (-(*idum) < 1) *idum=1;
		  else *idum = -(*idum);
                  for (j=NTAB+7;j>=0;j--) {
		        k=(*idum)/IQ;
                        *idum=IA*(*idum-k*IQ)-IR*k;
                        if (*idum < 0) *idum += IM;
                        if (j < NTAB) iv[j] = *idum;
					   }
	          iy=iv[0];
		   }
	          k=(*idum)/IQ;
	          *idum=IA*(*idum-k*IQ)-IR*k;
	          if (*idum < 0) *idum += IM;
	          j=iy/NDIV;
	          iy=iv[j];
	          iv[j] = *idum;
	          if ((temp=AM*iy) > RNMX) return RNMX;
	          else return temp;
	}
#undef IA
#undef IM
#undef AM
#undef IQ
#undef IR
#undef NTAB
#undef NDIV
#undef EPS
#undef RNMX

/*************************************************************************************/


#define NR_END 1

void nrerror(char error_text[])
/* Numerical Recipes standard error handler */
{
        printf("Numerical Recipes run-time error...\n");
        printf("%s\n",error_text);
        printf("...now exiting to system...\n");
        exit(1);
      }

float *vector(long nl, long nh)
/* allocate a float vector with subscript range v[nl..nh] */
{
        float *v;

        v=(float *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(float)));
        if (!v) nrerror("allocation failure in vector()");
        return (v-nl+NR_END);
      }

float **matrix(long nrl, long nrh, long ncl, long nch)
/* allocate a float matrix with subscript range m[nrl..nrh][ncl..nch] */
{
        long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
        float **m;

        /* allocate pointers to rows */
        m=(float **) malloc((size_t)((nrow+NR_END)*sizeof(float*)));
        if (!m) nrerror("allocation failure 1 in matrix()");
        m += NR_END;
        m -= nrl;

        /* allocate rows and set pointers to them */
        m[nrl]=(float *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(float)));
        if (!m[nrl]) nrerror("allocation failure 2 in matrix()");
        m[nrl] += NR_END;
        m[nrl] -= ncl;

        for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

        /* return pointer to array of pointers to rows */
        return (m);
      }

float ***f3tensor(long nrl, long nrh, long ncl, long nch, long ndl, long ndh)
     /* allocate a float 3tensor with range t[nrl..nrh][ncl..nch][ndl..ndh] */
{
        long i,j,nrow=nrh-nrl+1,ncol=nch-ncl+1,ndep=ndh-ndl+1;
        float ***t;

        /* allocate pointers to pointers to rows */
        t=(float ***) malloc((size_t)((nrow+NR_END)*sizeof(float**)));
        if (!t) nrerror("allocation failure 1 in f3tensor()");
        t += NR_END;
        t -= nrl;

        /* allocate pointers to rows and set pointers to them */
        t[nrl]=(float **) malloc((size_t)((nrow*ncol+NR_END)*sizeof(float*)));
        if (!t[nrl]) nrerror("allocation failure 2 in f3tensor()");
        t[nrl] += NR_END;
        t[nrl] -= ncl;

        /* allocate rows and set pointers to them */
        t[nrl][ncl]=(float *) malloc((size_t)((nrow*ncol*ndep+NR_END)*sizeof(float)));
        if (!t[nrl][ncl]) nrerror("allocation failure 3 in f3tensor()");
        t[nrl][ncl] += NR_END;
        t[nrl][ncl] -= ndl;

        for(j=ncl+1;j<=nch;j++) t[nrl][j]=t[nrl][j-1]+ndep;
        for(i=nrl+1;i<=nrh;i++) {
                t[i]=t[i-1]+ncol;
                t[i][ncl]=t[i-1][ncl]+ncol*ndep;
                for(j=ncl+1;j<=nch;j++) t[i][j]=t[i][j-1]+ndep;
        }

        /* return pointer to array of pointers to rows */
        return t;
}


void free_vector(float *v, long nl, long nh)
/* free a float vector allocated with vector() */
{
        nh++;
        free((char*) (v+nl-NR_END));
      }


void free_matrix(float **m, long nrl, long nrh, long ncl, long nch)
/* free a float matrix allocated by matrix() */
{
        nch++;nrh++;
        free((char*) (m[nrl]+ncl-NR_END));
        free((char*) (m+nrl-NR_END));
      }


int **imatrix(long nrl, long nrh, long ncl, long nch)
/* allocate a int matrix with subscript range m[nrl..nrh][ncl..nch] */
{
        long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
        int **m;

        /* allocate pointers to rows */
        m=(int **) malloc((size_t)((nrow+NR_END)*sizeof(int*)));
        if (!m) nrerror("allocation failure 1 in matrix()");
        m += NR_END;
        m -= nrl;


        /* allocate rows and set pointers to them */
        m[nrl]=(int *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(int)));
        if (!m[nrl]) nrerror("allocation failure 2 in matrix()");
        m[nrl] += NR_END;
        m[nrl] -= ncl;

        for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

        /* return pointer to array of pointers to rows */
        return m;
      }

int *ivector(long nl, long nh)
     /* allocate an int vector with subscript range v[nl..nh] */
{
        int *v;

        v=(int *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(int)));
        if (!v) nrerror("allocation failure in ivector()");
        return v-nl+NR_END;
}

void free_ivector(int *v, long nl, long nh)
     /* free an int vector allocated with ivector() */
{
        free((char*)(v+nl-NR_END));
}


void free_imatrix(int **m,long nrl,long nrh,long ncl,long nch)
/* free an int matrix allocated by imatrix() */
{
        nch++;nrh++;
        free((char*) (m[nrl]+ncl-NR_END));
        free((char*) (m+nrl-NR_END));
      }


void free_f3tensor(float ***t, long nrl, long nrh, long ncl, long nch,
        long ndl, long ndh)
     /* free a float f3tensor allocated by f3tensor() */
{
        free((char*) (t[nrl][ncl]+ndl-NR_END));
        free((char*) (t[nrl]+ncl-NR_END));
        free((char*) (t+nrl-NR_END));
}

complex **complexmatrix(long nrl, long nrh, long ncl, long nch)
/* allocate a complex matrix with subscript range m[nrl..nrh][ncl..nch] */
{
        long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
        complex **m;

        /* allocate pointers to rows */
        m=(complex **) malloc((size_t)((nrow+NR_END)*sizeof(complex*)));
        if (!m) nrerror("allocation failure 1 in matrix()");
        m += NR_END;
        m -= nrl;

        /* allocate rows and set pointers to them */
        m[nrl]=(complex *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(complex)));
        if (!m[nrl]) nrerror("allocation failure 2 in matrix()");
        m[nrl] += NR_END;
        m[nrl] -= ncl;

        for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

        /* return pointer to array of pointers to rows */
        return (m);
      }


complex *complexvector(long nl, long nh)
/* allocate a double vector with subscript range v[nl..nh] */
{
    complex *v;

    v=(complex *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(complex)));
    if (!v) nrerror("allocation failure in dvector()");
    return v-nl+NR_END;
}


void free_complexvector(complex *v, long nl, long nh)
{
        free((char*)(v+nl-NR_END));
}





